import {
	CanActivate,
	ExecutionContext,
	Injectable,
	UnauthorizedException,
} from '@nestjs/common';
import { GqlExecutionContext } from '@nestjs/graphql';
import { Request } from 'express';
import { AgentFlags } from '../resolvers/models/agent.model';

@Injectable()
export class GroupGuard implements CanActivate {
	canActivate(context: ExecutionContext) {
		const ctx = GqlExecutionContext.create(context);
		const req: Request = ctx.getContext().req;

		if (
			req.agent &&
			(req.agent.flags & AgentFlags.ADMIN) === AgentFlags.ADMIN
		)
			return true;

		if (!req.group) throw new UnauthorizedException('Not a group');

		return true;
	}
}
